@@ -4,16 +4,16 @@
 
 #include "CoderMixer2ST.h"
 
-namespace NCoderMixer2 {
+namespace NCoderMixer {
 
 CCoderMixer2ST::CCoderMixer2ST() {}
 
 CCoderMixer2ST::~CCoderMixer2ST(){ }
 
 HRESULT CCoderMixer2ST::SetBindInfo(const CBindInfo &bindInfo)
-{  
-  _bindInfo = bindInfo; 
-  return S_OK; 
+{
+  _bindInfo = bindInfo;
+  return S_OK;
 }
 
 void CCoderMixer2ST::AddCoderCommon(bool isMain)
@@ -37,27 +37,27 @@ void CCoderMixer2ST::AddCoder2(ICompressCoder2 *coder, bool isMain)
 void CCoderMixer2ST::ReInit() { }
 
 HRESULT CCoderMixer2ST::GetInStream(
-    ISequentialInStream **inStreams, const UInt64 **inSizes, 
+    ISequentialInStream **inStreams, const UInt64 **inSizes,
     UInt32 streamIndex, ISequentialInStream **inStreamRes)
 {
   CMyComPtr<ISequentialInStream> seqInStream;
   int i;
-  for(i = 0; i < _bindInfo.InStreams.Size(); i++)
+  for (i = 0; i < _bindInfo.InStreams.Size(); i++)
     if (_bindInfo.InStreams[i] == streamIndex)
     {
       seqInStream = inStreams[i];
       *inStreamRes = seqInStream.Detach();
-      return  S_OK;
+      return S_OK;
     }
   int binderIndex = _bindInfo.FindBinderForInStream(streamIndex);
   if (binderIndex < 0)
     return E_INVALIDARG;
 
   UInt32 coderIndex, coderStreamIndex;
-  _bindInfo.FindOutStream(_bindInfo.BindPairs[binderIndex].OutIndex, 
+  _bindInfo.FindOutStream(_bindInfo.BindPairs[binderIndex].OutIndex,
       coderIndex, coderStreamIndex);
 
-  CCoderInfo &coder = _coders[coderIndex];
+  CCoderInfo2 &coder = _coders[coderIndex];
   if (!coder.Coder)
     return E_NOTIMPL;
   coder.Coder.QueryInterface(IID_ISequentialInStream, &seqInStream);
@@ -86,27 +86,27 @@ HRESULT CCoderMixer2ST::GetInStream(
 }
 
 HRESULT CCoderMixer2ST::GetOutStream(
-    ISequentialOutStream **outStreams, const UInt64 **outSizes, 
+    ISequentialOutStream **outStreams, const UInt64 **outSizes,
     UInt32 streamIndex, ISequentialOutStream **outStreamRes)
 {
   CMyComPtr<ISequentialOutStream> seqOutStream;
   int i;
-  for(i = 0; i < _bindInfo.OutStreams.Size(); i++)
+  for (i = 0; i < _bindInfo.OutStreams.Size(); i++)
     if (_bindInfo.OutStreams[i] == streamIndex)
     {
       seqOutStream = outStreams[i];
       *outStreamRes = seqOutStream.Detach();
-      return  S_OK;
+      return S_OK;
     }
   int binderIndex = _bindInfo.FindBinderForOutStream(streamIndex);
   if (binderIndex < 0)
     return E_INVALIDARG;
 
   UInt32 coderIndex, coderStreamIndex;
-  _bindInfo.FindInStream(_bindInfo.BindPairs[binderIndex].InIndex, 
+  _bindInfo.FindInStream(_bindInfo.BindPairs[binderIndex].InIndex,
       coderIndex, coderStreamIndex);
 
-  CCoderInfo &coder = _coders[coderIndex];
+  CCoderInfo2 &coder = _coders[coderIndex];
   if (!coder.Coder)
     return E_NOTIMPL;
   coder.Coder.QueryInterface(IID_ISequentialOutStream, &seqOutStream);
@@ -136,9 +136,9 @@ HRESULT CCoderMixer2ST::GetOutStream(
     
 
 STDMETHODIMP CCoderMixer2ST::Code(ISequentialInStream **inStreams,
-      const UInt64 **inSizes, 
+      const UInt64 **inSizes,
       UInt32 numInStreams,
-      ISequentialOutStream **outStreams, 
+      ISequentialOutStream **outStreams,
       const UInt64 **outSizes,
       UInt32 numOutStreams,
       ICompressProgressInfo *progress)
@@ -148,7 +148,7 @@ STDMETHODIMP CCoderMixer2ST::Code(ISequentialInStream **inStreams,
     return E_INVALIDARG;
 
   // Find main coder
-  int _mainCoderIndex = -1; // FIXME - hide _mainCoderIndex
+  int _mainCoderIndex = -1;
   int i;
   for (i = 0; i < _coders.Size(); i++)
     if (_coders[i].IsMain)
@@ -169,7 +169,7 @@ STDMETHODIMP CCoderMixer2ST::Code(ISequentialInStream **inStreams,
  
   // _mainCoderIndex = 0;
   // _mainCoderIndex = _coders.Size() - 1;
-  CCoderInfo &mainCoder = _coders[_mainCoderIndex];
+  CCoderInfo2 &mainCoder = _coders[_mainCoderIndex];
 
   CObjectVector< CMyComPtr<ISequentialInStream> > seqInStreams;
   CObjectVector< CMyComPtr<ISequentialOutStream> > seqOutStreams;
@@ -198,7 +198,7 @@ STDMETHODIMP CCoderMixer2ST::Code(ISequentialInStream **inStreams,
   {
     if (i == _mainCoderIndex)
       continue;
-    CCoderInfo &coder = _coders[i];
+    CCoderInfo2 &coder = _coders[i];
     CMyComPtr<ICompressSetOutStreamSize> setOutStreamSize;
     coder.Coder.QueryInterface(IID_ICompressSetOutStreamSize, &setOutStreamSize);
     if (setOutStreamSize)
@@ -216,9 +216,9 @@ STDMETHODIMP CCoderMixer2ST::Code(ISequentialInStream **inStreams,
   else
   {
     RINOK(mainCoder.Coder2->Code(
-        &seqInStreamsSpec.Front(), 
+        &seqInStreamsSpec.Front(),
         &mainCoder.InSizePointers.Front(), mainCoder.NumInStreams,
-        &seqOutStreamsSpec.Front(), 
+        &seqOutStreamsSpec.Front(),
         &mainCoder.OutSizePointers.Front(), mainCoder.NumOutStreams,
         progress));
   }
@@ -236,4 +236,4 @@ UInt64 CCoderMixer2ST::GetWriteProcessedSize(UInt32 binderIndex) const
 }
 */
 
-}  
+}
