@@ -2,14 +2,18 @@
 
 #include "StdAfx.h"
 
-#include "Common/StringConvert.h"
-#include "Common/Wildcard.h"
+#include "../../../Common/StringConvert.h"
+#include "../../../Common/Wildcard.h"
 
-#include "Windows/PropVariant.h"
+#include "../../../Windows/FileName.h"
+#include "../../../Windows/FileDir.h"
+#include "../../../Windows/PropVariant.h"
 
 #include "../../PropID.h"
 
-#ifdef _WIN32
+#ifdef UNDER_CE
+#include "FSFolder.h"
+#else
 #include "FSDrives.h"
 #endif
 #include "LangUtils.h"
@@ -26,25 +30,24 @@ using namespace NFind;
 
 void CPanel::SetToRootFolder()
 {
-  _folder.Release();
+  ReleaseFolder();
   _library.Free();
   CRootFolder *rootFolderSpec = new CRootFolder;
-  _folder = rootFolderSpec;
+  SetNewFolder(rootFolderSpec);
   rootFolderSpec->Init();
 }
 
-HRESULT CPanel::BindToPath(const UString &fullPath, bool &archiveIsOpened, bool &encrypted)
+HRESULT CPanel::BindToPath(const UString &fullPath, const UString &arcFormat, bool &archiveIsOpened, bool &encrypted)
 {
   archiveIsOpened = false;
   encrypted = false;
-  CDisableTimerProcessing disableTimerProcessing1(*this);
-
-  printf("CPanel::BindToPath(%ls)\n",(const wchar_t *)fullPath);
+  CDisableTimerProcessing disableTimerProcessing(*this);
+  CDisableNotify disableNotify(*this);
 
   if (_parentFolders.Size() > 0)
   {
     const UString &virtPath = _parentFolders.Back().VirtualPath;
-    if (fullPath.Left(virtPath.Length()) == virtPath)
+    if (fullPath.IsPrefixedBy(virtPath))
     {
       for (;;)
       {
@@ -52,11 +55,11 @@ HRESULT CPanel::BindToPath(const UString &fullPath, bool &archiveIsOpened, bool
         HRESULT res = _folder->BindToParentFolder(&newFolder);
         if (!newFolder || res != S_OK)
           break;
-        _folder = newFolder;
+        SetNewFolder(newFolder);
       }
       UStringVector parts;
-      SplitPathToParts(fullPath.Mid(virtPath.Length()), parts);
-      for (int i = 0; i < parts.Size(); i++)
+      SplitPathToParts(fullPath.Ptr(virtPath.Len()), parts);
+      FOR_VECTOR (i, parts)
       {
         const UString &s = parts[i];
         if ((i == 0 || i == parts.Size() - 1) && s.IsEmpty())
@@ -65,7 +68,7 @@ HRESULT CPanel::BindToPath(const UString &fullPath, bool &archiveIsOpened, bool
         HRESULT res = _folder->BindToFolder(s, &newFolder);
         if (!newFolder || res != S_OK)
           break;
-        _folder = newFolder;
+        SetNewFolder(newFolder);
       }
       return S_OK;
     }
@@ -73,20 +76,20 @@ HRESULT CPanel::BindToPath(const UString &fullPath, bool &archiveIsOpened, bool
 
   CloseOpenFolders();
   UString sysPath = fullPath;
-  CFileInfoW fileInfo;
+  CFileInfo fileInfo;
   UStringVector reducedParts;
   while (!sysPath.IsEmpty())
   {
-    if (fileInfo.Find(sysPath))
+    if (fileInfo.Find(us2fs(sysPath)))
       break;
     int pos = sysPath.ReverseFind(WCHAR_PATH_SEPARATOR);
     if (pos < 0)
       sysPath.Empty();
     else
     {
-      if (reducedParts.Size() > 0 || pos < sysPath.Length() - 1)
-        reducedParts.Add(sysPath.Mid(pos + 1));
-      sysPath = sysPath.Left(pos);
+      if (reducedParts.Size() > 0 || pos < (int)sysPath.Len() - 1)
+        reducedParts.Add(sysPath.Ptr(pos + 1));
+      sysPath.DeleteFrom(pos);
     }
   }
   SetToRootFolder();
@@ -94,27 +97,24 @@ HRESULT CPanel::BindToPath(const UString &fullPath, bool &archiveIsOpened, bool
   if (sysPath.IsEmpty())
   {
     if (_folder->BindToFolder(fullPath, &newFolder) == S_OK)
-      _folder = newFolder;
+      SetNewFolder(newFolder);
   }
   else if (fileInfo.IsDir())
   {
     NName::NormalizeDirPathPrefix(sysPath);
     if (_folder->BindToFolder(sysPath, &newFolder) == S_OK)
-      _folder = newFolder;
+      SetNewFolder(newFolder);
   }
   else
   {
-    UString dirPrefix;
-    if (!NDirectory::GetOnlyDirPrefix(sysPath, dirPrefix))
-      dirPrefix.Empty();
-    if (_folder->BindToFolder(dirPrefix, &newFolder) == S_OK)
+    FString dirPrefix, fileName;
+    NDir::GetFullPathAndSplit(us2fs(sysPath), dirPrefix, fileName);
+    if (_folder->BindToFolder(fs2us(dirPrefix), &newFolder) == S_OK)
     {
-      _folder = newFolder;
+      SetNewFolder(newFolder);
       LoadFullPath();
-      UString fileName;
-      if (NDirectory::GetOnlyName(sysPath, fileName))
       {
-        HRESULT res = OpenItemAsArchive(fileName, encrypted);
+        HRESULT res = OpenItemAsArchive(fs2us(fileName), arcFormat, encrypted);
         if (res != S_FALSE)
         {
           RINOK(res);
@@ -132,7 +132,7 @@ HRESULT CPanel::BindToPath(const UString &fullPath, bool &archiveIsOpened, bool
             _folder->BindToFolder(reducedParts[i], &newFolder);
             if (!newFolder)
               break;
-            _folder = newFolder;
+            SetNewFolder(newFolder);
           }
         }
       }
@@ -143,9 +143,10 @@ HRESULT CPanel::BindToPath(const UString &fullPath, bool &archiveIsOpened, bool
 
 HRESULT CPanel::BindToPathAndRefresh(const UString &path)
 {
-  CDisableTimerProcessing disableTimerProcessing1(*this);
+  CDisableTimerProcessing disableTimerProcessing(*this);
+  CDisableNotify disableNotify(*this);
   bool archiveIsOpened, encrypted;
-  RINOK(BindToPath(path, archiveIsOpened, encrypted));
+  RINOK(BindToPath(path, UString(), archiveIsOpened, encrypted));
   RefreshListCtrl(UString(), -1, true, UStringVector());
   return S_OK;
 }
@@ -172,18 +173,18 @@ UString GetFolderPath(IFolderFolder *folder)
 void CPanel::LoadFullPath()
 {
   _currentFolderPrefix.Empty();
-  for (int i = 0; i < _parentFolders.Size(); i++)
+  FOR_VECTOR (i, _parentFolders)
   {
     const CFolderLink &folderLink = _parentFolders[i];
     _currentFolderPrefix += GetFolderPath(folderLink.ParentFolder);
-    _currentFolderPrefix += folderLink.ItemName;
+    _currentFolderPrefix += folderLink.RelPath;
     _currentFolderPrefix += WCHAR_PATH_SEPARATOR;
   }
   if (_folder)
     _currentFolderPrefix += GetFolderPath(_folder);
 }
 
-static int GetRealIconIndex(LPCWSTR path, DWORD attributes)
+static int GetRealIconIndex(CFSTR path, DWORD attributes)
 {
   int index = -1;
   if (GetRealIconIndex(path, attributes, index) != 0)
@@ -196,27 +197,44 @@ void CPanel::LoadFullPathAndShow()
   LoadFullPath();
   _appState->FolderHistory.AddString(_currentFolderPrefix);
 
+#ifdef _WIN32
   _headerComboBox.SetText(_currentFolderPrefix);
+#else
+  {
+    extern const TCHAR * nameWindowToUnix(const TCHAR * lpFileName);
+	UString tmp = nameWindowToUnix(_currentFolderPrefix);
+	_headerComboBox.SetText(tmp);
+  }	
+#endif
+
 #ifdef _WIN32 // FIXME
+  #ifndef UNDER_CE
+
   COMBOBOXEXITEM item;
   item.mask = 0;
 
   UString path = _currentFolderPrefix;
-  if (path.Length() >
+  if (path.Len() >
       #ifdef _WIN32
       3
       #else
       1
       #endif
-      && path[path.Length() - 1] == WCHAR_PATH_SEPARATOR)
-    path.Delete(path.Length() - 1);
+      && path.Back() == WCHAR_PATH_SEPARATOR)
+    path.DeleteBack();
 
-  CFileInfoW info;
   DWORD attrib = FILE_ATTRIBUTE_DIRECTORY;
-  if (info.Find(path))
-    attrib = info.Attrib;
-  
-  item.iImage = GetRealIconIndex(path, attrib);
+
+  // GetRealIconIndex is slow for direct DVD/UDF path. So we use dummy path
+  if (path.IsPrefixedBy(L"\\\\.\\"))
+    path = L"_TestFolder_";
+  else
+  {
+    CFileInfo fi;
+    if (fi.Find(us2fs(path)))
+      attrib = fi.Attrib;
+  }
+  item.iImage = GetRealIconIndex(us2fs(path), attrib);
 
   if (item.iImage >= 0)
   {
@@ -225,17 +243,19 @@ void CPanel::LoadFullPathAndShow()
   }
   item.iItem = -1;
   _headerComboBox.SetItem(&item);
+  
+  #endif
 #endif
 
   RefreshTitle();
 }
 
-#ifdef _WIN32
+#ifndef UNDER_CE
 LRESULT CPanel::OnNotifyComboBoxEnter(const UString &s)
 {
   if (BindToPathAndRefresh(GetUnicodeString(s)) == S_OK)
   {
-    PostMessage(kSetFocusToListView);
+    // FIXME PostMessage(kSetFocusToListView);
     return TRUE;
   }
   return FALSE;
@@ -246,7 +266,7 @@ bool CPanel::OnNotifyComboBoxEndEdit(PNMCBEENDEDITW info, LRESULT &result)
   if (info->iWhy == CBENF_ESCAPE)
   {
     _headerComboBox.SetText(_currentFolderPrefix);
-    PostMessage(kSetFocusToListView);
+    // FIXME PostMessage(kSetFocusToListView);
     result = FALSE;
     return true;
   }
@@ -269,6 +289,7 @@ bool CPanel::OnNotifyComboBoxEndEdit(PNMCBEENDEDITW info, LRESULT &result)
   }
   return false;
 }
+#endif
 
 #ifndef _UNICODE
 bool CPanel::OnNotifyComboBoxEndEdit(PNMCBEENDEDIT info, LRESULT &result)
@@ -300,8 +321,19 @@ bool CPanel::OnNotifyComboBoxEndEdit(PNMCBEENDEDIT info, LRESULT &result)
 }
 #endif
 
+
 void CPanel::AddComboBoxItem(const UString &name, int iconIndex, int indent, bool addToList)
 {
+  #if 1 // FIXME #ifdef UNDER_CE
+
+  UString s;
+  iconIndex = iconIndex;
+  for (int i = 0; i < indent; i++)
+    s += L"  ";
+  _headerComboBox.AddString(s + name);
+  
+  #else
+  
   COMBOBOXEXITEMW item;
   item.mask = CBEIF_TEXT | CBEIF_INDENT;
   item.iSelectedImage = item.iImage = iconIndex;
@@ -311,6 +343,9 @@ void CPanel::AddComboBoxItem(const UString &name, int iconIndex, int indent, boo
   item.iIndent = indent;
   item.pszText = (LPWSTR)(LPCWSTR)name;
   _headerComboBox.InsertItem(&item);
+  
+  #endif
+
   if (addToList)
     ComboBoxPaths.Add(name);
 }
@@ -321,6 +356,7 @@ extern UString RootFolder_GetName_Documents(int &iconIndex);
 
 bool CPanel::OnComboBoxCommand(UINT code, LPARAM /* param */, LRESULT &result)
 {
+#ifdef _WIN32 // FIXME
   result = FALSE;
   switch(code)
   {
@@ -329,54 +365,52 @@ bool CPanel::OnComboBoxCommand(UINT code, LPARAM /* param */, LRESULT &result)
       ComboBoxPaths.Clear();
       _headerComboBox.ResetContent();
       
-      int iconIndex;
-      UString name;
-
-      int i;
+      unsigned i;
       UStringVector pathParts;
       
       SplitPathToParts(_currentFolderPrefix, pathParts);
       UString sumPass;
+      if (!pathParts.IsEmpty())
+        pathParts.DeleteBack();
       for (i = 0; i < pathParts.Size(); i++)
       {
         UString name = pathParts[i];
-        if (name.IsEmpty())
-          continue;
         sumPass += name;
-        UString curName = sumPass;
-        if (i == 0)
-          curName += WCHAR_PATH_SEPARATOR;
-        CFileInfoW info;
-        DWORD attrib = FILE_ATTRIBUTE_DIRECTORY;
-        if (info.Find(sumPass))
-          attrib = info.Attrib;
         sumPass += WCHAR_PATH_SEPARATOR;
-        AddComboBoxItem(name, GetRealIconIndex(curName, attrib), i, false);
+        CFileInfo info;
+        DWORD attrib = FILE_ATTRIBUTE_DIRECTORY;
+        if (info.Find(us2fs(sumPass)))
+          attrib = info.Attrib;
+        AddComboBoxItem(name.IsEmpty() ? L"\\" : name, GetRealIconIndex(us2fs(sumPass), attrib), i, false);
         ComboBoxPaths.Add(sumPass);
       }
 
+      #ifndef UNDER_CE
+
+      int iconIndex;
+      UString name;
       name = RootFolder_GetName_Documents(iconIndex);
       AddComboBoxItem(name, iconIndex, 0, true);
 
       name = RootFolder_GetName_Computer(iconIndex);
       AddComboBoxItem(name, iconIndex, 0, true);
         
-      UStringVector driveStrings;
+      FStringVector driveStrings;
       MyGetLogicalDriveStrings(driveStrings);
       for (i = 0; i < driveStrings.Size(); i++)
       {
-        UString s = driveStrings[i];
-        ComboBoxPaths.Add(s);
+        FString s = driveStrings[i];
+        ComboBoxPaths.Add(fs2us(s));
         int iconIndex = GetRealIconIndex(s, 0);
-        if (s.Length() > 0 && s[s.Length() - 1] == WCHAR_PATH_SEPARATOR)
-          s.Delete(s.Length() - 1);
-        AddComboBoxItem(s, iconIndex, 1, false);
+        if (s.Len() > 0 && s.Back() == FCHAR_PATH_SEPARATOR)
+          s.DeleteBack();
+        AddComboBoxItem(fs2us(s), iconIndex, 1, false);
       }
 
       name = RootFolder_GetName_Network(iconIndex);
       AddComboBoxItem(name, iconIndex, 0, true);
 
-      // UStringVector strings; _appState->FolderHistory.GetList(strings);
+      #endif
     
       return false;
     }
@@ -389,10 +423,13 @@ bool CPanel::OnComboBoxCommand(UINT code, LPARAM /* param */, LRESULT &result)
       {
         UString pass = ComboBoxPaths[index];
         _headerComboBox.SetCurSel(-1);
-        _headerComboBox.SetText(pass); // it's fix for seclecting by mouse.
+        // _headerComboBox.SetText(pass); // it's fix for seclecting by mouse.
         if (BindToPathAndRefresh(pass) == S_OK)
         {
           PostMessage(kSetFocusToListView);
+          #ifdef UNDER_CE
+          PostMessage(kRefresh_HeaderComboBox);
+          #endif
           return true;
         }
       }
@@ -412,12 +449,14 @@ bool CPanel::OnComboBoxCommand(UINT code, LPARAM /* param */, LRESULT &result)
     }
     */
   }
+#endif
   return false;
 }
 
-bool CPanel::OnNotifyComboBox(LPNMHDR header, LRESULT &result)
+bool CPanel::OnNotifyComboBox(LPNMHDR NON_CE_VAR(header), LRESULT & NON_CE_VAR(result))
 {
-  switch(header->code)
+  #ifndef UNDER_CE
+  switch (header->code)
   {
     case CBEN_BEGINEDIT:
     {
@@ -425,29 +464,31 @@ bool CPanel::OnNotifyComboBox(LPNMHDR header, LRESULT &result)
       _panelCallback->PanelWasFocused();
       break;
     }
+#ifdef _WIN32
     #ifndef _UNICODE
     case CBEN_ENDEDIT:
     {
       return OnNotifyComboBoxEndEdit((PNMCBEENDEDIT)header, result);
     }
     #endif
+#endif
     case CBEN_ENDEDITW:
     {
       return OnNotifyComboBoxEndEdit((PNMCBEENDEDITW)header, result);
     }
   }
+  #endif
   return false;
 }
-#endif
 
 
 void CPanel::FoldersHistory()
 {
   CListViewDialog listViewDialog;
   listViewDialog.DeleteIsAllowed = true;
-  listViewDialog.Title = LangString(IDS_FOLDERS_HISTORY, 0x03020260);
+  LangString(IDS_FOLDERS_HISTORY, listViewDialog.Title);
   _appState->FolderHistory.GetList(listViewDialog.Strings);
-  if (listViewDialog.Create(GetParent()) == IDCANCEL)
+  if (listViewDialog.Create(GetParent()) != IDOK)
     return;
   UString selectString;
   if (listViewDialog.StringsWereChanged)
@@ -469,28 +510,27 @@ void CPanel::FoldersHistory()
 
 void CPanel::OpenParentFolder()
 {
-printf("CPanel::OpenParentFolder\n");
   LoadFullPath(); // Maybe we don't need it ??
   UString focucedName;
-  if (!_currentFolderPrefix.IsEmpty())
+  if (!_currentFolderPrefix.IsEmpty() &&
+      _currentFolderPrefix.Back() == WCHAR_PATH_SEPARATOR)
   {
-    UString string = _currentFolderPrefix;
-    string.Delete(string.Length() - 1);
-    int pos = string.ReverseFind(WCHAR_PATH_SEPARATOR);
-    if (pos < 0)
-      pos = 0;
-    else
-      pos++;
-    focucedName = string.Mid(pos);
+    focucedName = _currentFolderPrefix;
+    focucedName.DeleteBack();
+    if (focucedName != L"\\\\.")
+    {
+      int pos = focucedName.ReverseFind(WCHAR_PATH_SEPARATOR);
+      if (pos >= 0)
+        focucedName.DeleteFrontal(pos + 1);
+    }
   }
 
-  printf("CPanel::OpenParentFolder focucedName=%ls\n",(const wchar_t *)focucedName);
-
-  CDisableTimerProcessing disableTimerProcessing1(*this);
+  CDisableTimerProcessing disableTimerProcessing(*this);
+  CDisableNotify disableNotify(*this);
   CMyComPtr<IFolderFolder> newFolder;
   _folder->BindToParentFolder(&newFolder);
   if (newFolder)
-    _folder = newFolder;
+    SetNewFolder(newFolder);
   else
   {
     if (_parentFolders.IsEmpty())
@@ -501,12 +541,12 @@ printf("CPanel::OpenParentFolder\n");
     }
     else
     {
-      _folder.Release();
+      ReleaseFolder();
       _library.Free();
       CFolderLink &link = _parentFolders.Back();
-      _folder = link.ParentFolder;
+      SetNewFolder(link.ParentFolder);
       _library.Attach(link.Library.Detach());
-      focucedName = link.ItemName;
+      focucedName = link.RelPath;
       if (_parentFolders.Size() > 1)
         OpenParentArchiveFolder();
       _parentFolders.DeleteBack();
@@ -523,32 +563,30 @@ printf("CPanel::OpenParentFolder\n");
   LoadFullPath();
   // ::SetCurrentDirectory(::_currentFolderPrefix);
   RefreshListCtrl(focucedName, -1, true, selectedItems);
-  _listView.EnsureVisible(_listView.GetFocusedItem(), false);
-  RefreshStatusBar();
-
-  printf("CPanel::OpenParentFolder-end\n");
+  // _listView.EnsureVisible(_listView.GetFocusedItem(), false);
 }
 
 void CPanel::CloseOpenFolders()
 {
   while (_parentFolders.Size() > 0)
   {
-    _folder.Release();
+    ReleaseFolder();
     _library.Free();
-    _folder = _parentFolders.Back().ParentFolder;
+    SetNewFolder(_parentFolders.Back().ParentFolder);
     _library.Attach(_parentFolders.Back().Library.Detach());
     if (_parentFolders.Size() > 1)
       OpenParentArchiveFolder();
     _parentFolders.DeleteBack();
   }
   _flatMode = _flatModeForDisk;
-  _folder.Release();
+  ReleaseFolder();
   _library.Free();
 }
 
 void CPanel::OpenRootFolder()
 {
-  CDisableTimerProcessing disableTimerProcessing1(*this);
+  CDisableTimerProcessing disableTimerProcessing(*this);
+  CDisableNotify disableNotify(*this);
   _parentFolders.Clear();
   SetToRootFolder();
   RefreshListCtrl(UString(), -1, true, UStringVector());
@@ -565,15 +603,17 @@ void CPanel::OpenRootFolder()
 
 void CPanel::OpenDrivesFolder()
 {
-#ifdef _WIN32
   CloseOpenFolders();
-  CFSDrives *fsFolderSpec = new CFSDrives;
-  _folder = fsFolderSpec;
-  fsFolderSpec->Init();
+  #ifdef UNDER_CE
+  NFsFolder::CFSFolder *folderSpec = new NFsFolder::CFSFolder;
+  SetNewFolder(folderSpec);
+  folderSpec->InitToRoot();
+  #else
+  CFSDrives *folderSpec = new CFSDrives;
+  SetNewFolder(folderSpec);
+  folderSpec->Init();
+  #endif
   RefreshListCtrl();
-#else
-  printf("CPanel::OpenDrivesFolder : FIXME\n");
-#endif
 }
 
 void CPanel::OpenFolder(int index)
@@ -587,11 +627,9 @@ void CPanel::OpenFolder(int index)
   _folder->BindToFolder(index, &newFolder);
   if (!newFolder)
     return;
-  _folder = newFolder;
+  SetNewFolder(newFolder);
   LoadFullPath();
-  // ::SetCurrentDirectory(::_currentFolderPrefix);
   RefreshListCtrl();
-  UINT state = LVIS_SELECTED;
-  _listView.SetItemState(_listView.GetFocusedItem(), state, state);
+  _listView.SetItemState_Selected(_listView.GetFocusedItem());
   _listView.EnsureVisible(_listView.GetFocusedItem(), false);
 }
